Compute the empirical detection/rejection rate for Type I errors and Power
Source:R/summary_functions.R
EDR.RdComputes the detection/rejection rate for determining empirical Type I error and power rates using information from p-values.
Arguments
- p
a
numericvector ormatrix/data.frameof p-values from the desired statistical estimator. If amatrix, each statistic must be organized by column, where the number of rows is equal to the number of replications- alpha
the detection threshold (typical values are .10, .05, and .01). Default is .05
- unname
logical; apply
unnameto the results to remove any variable names?
References
Chalmers, R. P., & Adkins, M. C. (2020). Writing Effective and Reliable Monte Carlo Simulations
with the SimDesign Package. The Quantitative Methods for Psychology, 16(4), 248-280.
doi:10.20982/tqmp.16.4.p248
Sigal, M. J., & Chalmers, R. P. (2016). Play it again: Teaching statistics with Monte
Carlo simulation. Journal of Statistics Education, 24(3), 136-156.
doi:10.1080/10691898.2016.1246953
Author
Phil Chalmers rphilip.chalmers@gmail.com