Convenience function that switches between readRDS() and qs2::qs_read() for functions saved with SimDesign. By convention, objects saved with the extension .rds are read as R binary files and typically reflect the final object from runSimulation or runArraySimulation, while files without an extension are read using qs2 (most often temporary files or results written to an associated sub-directory).

SimRead(filename)

Arguments

filename

name of the file to read in

Value

the R binary object