Learn R Programming

SimDesign (version 2.25)

SimErrors: Extract Simulation Errors

Description

Extractor function in situations where runSimulation returned a simulation with detected ERRORS.

Usage

SimErrors(obj, seeds = FALSE, subset = TRUE)

Arguments

obj

object returned from runSimulation containing an ERRORS column

seeds

logical; locate .Random.seed state that caused the error message?

subset

logical; take a subset of the design object showing only conditions that returned errors?

Author

Phil Chalmers rphilip.chalmers@gmail.com

References

Chalmers, R. P., & Adkins, M. C. (2020). Writing Effective and Reliable Monte Carlo Simulations with the SimDesign Package. The Quantitative Methods for Psychology, 16(4), 248-280. tools:::Rd_expr_doi("10.20982/tqmp.16.4.p248")

Sigal, M. J., & Chalmers, R. P. (2016). Play it again: Teaching statistics with Monte Carlo simulation. Journal of Statistics Education, 24(3), 136-156. tools:::Rd_expr_doi("10.1080/10691898.2016.1246953")

See Also

SimWarnings, SimExtract